package expr

import (
	"github.com/z7zmey/php-parser/meta"
	"github.com/z7zmey/php-parser/node"
	"github.com/z7zmey/php-parser/position"
	"github.com/z7zmey/php-parser/walker"
)

// Print node
type Print struct {
	Meta     []meta.Meta
	Position *position.Position
	Expr     node.Node
}

// NewPrint node constructor
func NewPrint(Expression node.Node) *Print {
	return &Print{
		Expr: Expression,
	}
}

// SetPosition sets node position
func (n *Print) SetPosition(p *position.Position) {
	n.Position = p
}

// GetPosition returns node positions
func (n *Print) GetPosition() *position.Position {
	return n.Position
}

func (n *Print) AddMeta(m []meta.Meta) {
	n.Meta = append(n.Meta, m...)
}

func (n *Print) GetMeta() []meta.Meta {
	return n.Meta
}

// Attributes returns node attributes as map
func (n *Print) Attributes() map[string]interface{} {
	return nil
}

// Walk traverses nodes
// Walk is invoked recursively until v.EnterNode returns true
func (n *Print) Walk(v walker.Visitor) {
	if v.EnterNode(n) == false {
		return
	}

	if n.Expr != nil {
		v.EnterChildNode("Expr", n)
		n.Expr.Walk(v)
		v.LeaveChildNode("Expr", n)
	}

	v.LeaveNode(n)
}
